/***********************************************************************************
  Oscar Martinez Valpuesta
  Practica 2 de Procesadores del Lenguaje I
  Objetivo: Implementar un analizador lexico en lex de un subconjunto del
	lenguaje C.
***********************************************************************************/
%{
#include "symbol.c"
const N = 7;	/* n de palabras reservadas */
%}
digito [0-9]
letra [a-zA-Z]
identificador {letra}({letra}|{digito}|_){0,31}
especial ";"|"("|")"|"{"|"}"|"+"|"-"|"/"|"%"|"*"|"<"|">"|"&"|"^"|"|"|"="|","
%x COMENTARIO

%%
 /* patron - accion */
{identificador}	{int reservada = binary(0,N-1);
		if (reservada != 0)
		   return reservada;		/* palabra reservada */
		 else {
		   yylval.ptr = lookUp(yytext);
	                    return ID;
		 }}
{digito}+	{yylval.value = atoi(yytext);
		 return CONST;}
{especial}	return yytext[0];
[\n] ;		/* evitamos lineas en blanco en pantalla */
[ ]+|[\t]+	;	/* evitamos espacios en blanco y tabuladores */
"/*"		BEGIN(COMENTARIO);
<COMENTARIO>[^*\n]*        /* pasamos por alto cualquier cosa que no sea '*' */
<COMENTARIO>"*"+[^*/\n]*   /* eliminamos todos los '*' no seguidos de '/' */
<COMENTARIO>"*"+"/"        BEGIN(INITIAL);
%%

static struct rwtable        /* reserved word table */
{
  char *name;               /* representation */
  int  token;                /* yylex() value  */
} reservedWords[7];           /* sorted */


/***********************************************************************************/ 
/* Procedimiento que crea la tabla de palabras reservadas. Esta tabla esta
ordenada alfabeticamente */
void CrearRWtable() {
  
  reservedWords[0].name = "break";
  reservedWords[0].token = BREAK;

  reservedWords[1].name = "continue";
  reservedWords[1].token = CONTINUE;

  reservedWords[2].name = "else";
  reservedWords[2].token = ELSE;

  reservedWords[3].name = "if";
  reservedWords[3].token = IF;

  reservedWords[4].name = "int";
  reservedWords[4].token = INT;

  reservedWords[5].name = "return";
  reservedWords[5].token = RETURN;

  reservedWords[6].name = "while";
  reservedWords[6].token = WHILE;
}

/***********************************************************************************/ 
 /*
Busqueda binaria en la tabla de palabras reservadas */

int binary(int inf, int sup) {
int mitad;

  if ((inf == sup) && (strcmp(yytext,reservedWords[inf].name) != 0))
     return 0;		/* no se ha encontrado */
  else {
    mitad = (int)(inf+sup) / 2;
    if (strcmp(yytext, reservedWords[mitad].name)==0) 	/* encontrado */
      return (reservedWords[mitad].token);
    else
      if (strcmp(yytext,reservedWords[mitad].name) > 0) 
        return (binary(mitad+1, sup));      
      else 
        return (binary(inf, mitad-1));
  }
}
/***********************************************************************************/

main( int argc, char ** argv )
{
int tok = 0;
FILE *file;

  /* comprobacion de parametros y apertura del fichero */
  if (argc > 1)
{
    file = fopen( argv[1], "r");
    if ( !file )
{
      fprintf(stderr, "no se puede abrir %s\n", argv[1] );
      exit(1);
    }
  } /* if */
  else {
    printf("Forma de uso: a.out fichero.c\n");
    exit (1);
  }

  yyin = file;

  /* Creamos la tabla de palabras reservadas */
  CrearRWtable();

  /* Creamos la tabla de simbolos */
  init();

  while ( (tok = yylex()) != 0 )
{
    switch ( tok )
{
      case CONST    : printf( "%d: CONST %d\n", yylineno, yylval.value );
                      break;
      case ID       : printf( "%d: ID %s\n", yylineno, yylval.ptr->name);
                      break;
      case IF       : printf( "%d: IF\n", yylineno);
                      break;
      case INT      : printf( "%d: INT\n", yylineno);
                      break;
      case ELSE     : printf( "%d: ELSE\n", yylineno);
                      break;
      case WHILE    : printf( "%d: WHILE\n", yylineno);
                      break;
      case BREAK    : printf( "%d: BREAK\n", yylineno);
                      break;
      case CONTINUE : printf( "%d: CONTINUE\n", yylineno);
                      break;
      case RETURN   : printf( "%d: RETURN\n", yylineno);
                      break;
      case GE       : /* >= */
                      printf( "%d: >=\n", yylineno);
                      break;
      case LE       : /* <= */
                      printf( "%d: <=\n", yylineno);
                      break;
      case EQ       : /* == */
                      printf( "%d: ==\n", yylineno);
                      break;
      case NE       : /* != */
                      printf( "%d: !=\n", yylineno);
                      break;
      case PE       : /* += */
                      printf( "%d: +=\n", yylineno);
                      break;
      case ME       : /* -= */
                      printf( "%d: -=\n", yylineno);
                      break;
      case TE       : /* *= */
                      printf( "%d: *=\n", yylineno);
                      break;
      case DE       : /* /= */
                      printf( "%d: /=\n", yylineno);
                      break;
      case RE       : /* %= */
                      printf( "%d: %=\n", yylineno);
                      break;
      case PP       : /* ++ */
                      printf( "%d: ++\n", yylineno);
                      break;
      case MM       : /* -- */
                      printf( "%d: --\n", yylineno);
                      break;
       case ';'	:      case '('	:       case ')':
 
      case '{'	:      case '}'	:      case '+' :
      case '-' 	:      case '*'	:      case '/' :
      case '%'	:      case '<'	:      case '>' :
      case '&' 	:      case '^'	:      case '|'	:
      case '='	:
      case ','	: printf( "%d: %c\n", yylineno, tok);
	          break;
      default	: printf( "%d: Unkown Character %c\n", yylineno, tok );
	          break;
    }; /* switch */
  } /*while */;
} /* main */

yywrap() {
  return 1;
}

