/********************************************************************************
   Modulo de definicion de la tabla de simbolos y rutinas necesarias para su
   manipulacion.
********************************************************************************/
#include <stdlib.h>
#include "y.tab.h"

struct entry {
  char *name;
  struct entry *next;
};

struct entry *symtab;      /* symbol table */

static struct rwtable        /* reserved word table */
{
  char *name;               /* representation */
  int  token;                /* yylex() value  */
} reservedWords[7];           /* sorted */

/********************************************************************************/ 
/*Procedimiento que muestra el error s en pantalla y termina la ejecucion */ 
void Mi_Error( char *s ){ 

  printf("%s\n", s);
  exit (0);		      
}/* error */ 

/*******************************************************************************/
/*Inicializacion de la tabla de simbolos */
void init()
{

  symtab = (struct entry *)malloc(sizeof(struct entry));
  if (symtab == NULL) 
    Mi_Error("ERROR, no se ha podido dar memoria a la tabla de simbolos");
  symtab->next = NULL;
} /* init */


/********************************************************************************/
/*Aade una nueva estructura a la lista y devuelve el top de la pila */
struct entry *create( char *name )
{
struct entry *nueva;

  nueva = (struct entry *)malloc(sizeof(struct entry));
  if (nueva == NULL)
    Mi_Error("ERROR, no hay memoria suficiente para una entrada en la tabla de simbolos");
  
  nueva->name = name;
  nueva->next = symtab;
  return nueva;
} /* create */

/********************************************************************************/  
/*
Busca una entrada en la lista con el nombre name y devuelve un puntero
a esa entrada. Si no se encuentra se devuelve un puntero nulo */
struct entry *find( char *name)
{
struct entry *aux;

  aux = symtab;
  while (aux->next != NULL) 
    if (strcmp(name, aux->name))
	return aux;

  return NULL;               /* no se ha encontrado */
}/* find */

/*
Funcion que mira si un determinado nombre esta en la tabla. Si esta devuelve
   un puntero a ese registro. si no esta lo inserta y devuelve un puntero al 
   nuevo registro */
struct entry *lookUp( char *s ){
struct entry *aux;

  aux = find(s);  
  if (aux == NULL)
    return (create(s));
  else
    return (aux);

} /* lookUp */

/* Procedimiento que crea la tabla de palabras reservadas. Esta tabla esta
ordenada alfabeticamente */
void CrearRWtable() {
  
  reservedWords[0].name = "break";
  reservedWords[0].token = BREAK;

  reservedWords[1].name = "continue";
  reservedWords[1].token = CONTINUE;

  reservedWords[2].name = "else";
  reservedWords[2].token = ELSE;

  reservedWords[3].name = "if";
  reservedWords[3].token = IF;

  reservedWords[4].name = "int";
  reservedWords[4].token = INT;

  reservedWords[5].name = "return";
  reservedWords[5].token = RETURN;

  reservedWords[6].name = "while";
  reservedWords[6].token = WHILE;
}

/***********************************************************************************/ 
 /*Busqueda binaria en la tabla de palabras reservadas */
int binary(int inf, int sup) {
int mitad;

  if ((inf == sup) && (strcmp(yytext,reservedWords[inf].name) != 0))
     return 0;		/* no se ha encontrado */
  else {
    mitad = (int)(inf+sup) / 2;
    if (strcmp(yytext, reservedWords[mitad].name)==0) 	/* encontrado */
      return (reservedWords[mitad].token);
    else
      if (strcmp(yytext,reservedWords[mitad].name) > 0) 
        return (binary(mitad+1, sup));      
      else 
        return (binary(inf, mitad-1));
  }
}
